import React, { PureComponent, Fragment } from 'react';
import TabContent from 'reactstrap/lib/TabContent';
import TabPane from 'reactstrap/lib/TabPane';
import Nav from 'reactstrap/lib/Nav';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { F, path, gt, divide, compose, __, prop } from 'ramda';
import { fromEvent } from 'rxjs/observable/fromEvent';
import { map } from 'rxjs/operators/map';
import { debounceTime } from 'rxjs/operators/debounceTime';
import { TabLabel } from './tab-label/tab-label';
import { NavWrapper } from './nav-wrapper/nav-wrapper';
import { IconArrowRight } from '../icons/icon-arrow-right/icon-arrow-right';
import { isMobile } from '../../helpers/ua';

const SCROLL_VALUE = 2.5;

export const getOffsetWidthFromRef = prop('offsetWidth');
export const getScrollWidthFromRef = prop('scrollWidth');
export const getOffsetFromWidth = compose(
  divide(__, SCROLL_VALUE),
  getOffsetWidthFromRef,
);

export class Tabs extends PureComponent {
  static propTypes = {
    activeTab: PropTypes.oneOfType([
      PropTypes.string,
      PropTypes.number,
    ]),
    children: PropTypes.node.isRequired,
    upper: PropTypes.bool,
    border: PropTypes.bool,
    flexItems: PropTypes.bool,
    onTabSelect: PropTypes.func,
    className: PropTypes.string,
    tabClass: PropTypes.string,
    contentClass: PropTypes.string,
  };

  static defaultProps = {
    activeTab: 0,
    upper: false,
    border: false,
    flexItems: false,
    className: null,
    tabClass: null,
    contentClass: null,
    onTabSelect: F,
  };

  constructor(props) {
    super(props);

    let { activeTab } = this.props;
    const firstTabId = path(['0', 'props', 'tabId'], this.props.children);

    if (firstTabId && !activeTab) {
      activeTab = firstTabId;
    }

    this.state = {
      activeTab,
      offset: 0,
      isSlideLeft: false,
      isSlideRight: false,
    };
  }

  isMaxOffset = false;

  componentDidMount() {
    if (!this.props.border) {
      return;
    }

    this.isSlideAvailable$ = fromEvent(window, 'resize')
      .pipe(
        debounceTime(200),
        map(this.onResize),
      ).subscribe();

    this.isSlide();
  }

  componentDidUpdate() {
    if (this.props.border) {
      this.isSlide();
    }
  }

  componentWillUnmount() {
    if (this.props.border) {
      this.isSlideAvailable$.unsubscribe();
    }
  }

  onResize = () => {
    this.isSlide();

    if (this.isMaxOffset) {
      this.slideRight();
    }
  }

  isSlide = () => {
    const { offset } = this.state;

    this.setState({
      isSlideLeft: this.isSlideLeft(offset),
      isSlideRight: this.isSlideRight(offset),
    });
  }

  getContainerRef = (node) => {
    this.containerRef = node;
  }

  getInnerRef = (node) => {
    this.innerRef = node;
  }

  getMaxScrollValue = () => {
    const containerWidth = getOffsetWidthFromRef(this.containerRef);
    const scrollWidth = getScrollWidthFromRef(this.innerRef);
    return containerWidth - scrollWidth;
  }

  isSlideLeft = gt(0);
  isSlideRight = (offset) => {
    const containerWidth = getOffsetWidthFromRef(this.containerRef);
    const scrollWidth = getScrollWidthFromRef(this.innerRef);

    if (!containerWidth || !scrollWidth) {
      return false;
    }

    return scrollWidth > containerWidth && offset > this.getMaxScrollValue();
  }

  getNextOffset = (prevState, dir) => {
    const offset = getOffsetFromWidth(this.containerRef);
    return this.prepareNextOffset(prevState.offset + (offset * dir));
  }

  prepareNextOffset = (offset) => {
    const maxScrollValue = this.getMaxScrollValue();
    let res = offset;

    this.isMaxOffset = false;

    if (offset <= maxScrollValue) {
      res = maxScrollValue;
      this.isMaxOffset = true;
    } else if (offset > 0) {
      res = 0;
    }

    return res;
  }

  slide = (dir) => {
    this.setState(prevState => ({
      offset: this.getNextOffset(prevState, dir),
    }));
  }

  slideRight = () => {
    this.slide(-1);
  }

  slideLeft = () => {
    this.slide(1);
  }

  onWheel = (e) => {
    if (!isMobile()) {
      return;
    }

    e.preventDefault();

    const { deltaX } = e;
    const { offset } = this.state;

    if ((deltaX < 0 && !this.isSlideLeft(offset)) || (deltaX > 0 && !this.isSlideRight(offset))) {
      return;
    }

    requestAnimationFrame(() => this.setState(prevState => ({
      offset: this.prepareNextOffset(prevState.offset + -deltaX),
    })));
  }

  toggle = (tabId) => {
    this.setState({ activeTab: tabId });
    this.props.onTabSelect(tabId);
  }

  renderTitles() {
    const {
      upper,
      border,
      flexItems,
      className,
      tabClass,
    } = this.props;
    const { offset, isSlideLeft, isSlideRight } = this.state;
    const classNames = classnames(
      'navbar-nav flex-row',
      {
        'tabs-upper': upper,
        'tabs-border': border,
        'tabs-flex-items': flexItems,
      }
    );

    return (
      <NavWrapper className={className}>
        <div className={classnames('nav-tabs-wrapper', { 'nav-tabs-wrapper-border': border })} ref={this.getContainerRef} onWheel={this.onWheel}>
          <div style={{ transform: `translate3d(${offset}px, 0, 0)` }} className="nav-tabs-inner" ref={this.getInnerRef}>
            <Nav tabs className={classNames} >
              {this.props.children.map((child, i) => {
                const { label, tabId } = child.props;
                const currentTabId = tabId || i;

                return label ? (
                  <TabLabel
                    key={label}
                    label={label}
                    tabId={currentTabId}
                    isActive={this.state.activeTab === currentTabId}
                    className={tabClass}
                    onClick={this.toggle}
                  />
                ) : React.cloneElement(child); // For cases when we need custom element in navbar (along-with TabLabel)
              })}
            </Nav>
          </div>

          {border && isSlideLeft &&
            <div
              role="button"
              tabIndex="0"
              className="tabs-arrow tabs-arrow-left h-100"
              onClick={this.slideLeft}
              onKeyPress={this.slideLeft}
            >
              <IconArrowRight />
            </div>}

          {border && isSlideRight &&
            <div
              role="button"
              tabIndex="0"
              className="tabs-arrow tabs-arrow-right h-100"
              onClick={this.slideRight}
              onKeyPress={this.slideRight}
            >
              <IconArrowRight />
            </div>}
        </div>
      </NavWrapper>
    );
  }

  render() {
    const { children, contentClass } = this.props;

    return (
      <Fragment>
        {this.renderTitles()}
        <TabContent className={contentClass} activeTab={this.state.activeTab}>
          {children.map((child, i) => {
            const { tabId, label, className } = child.props;
            return label ?
              React.cloneElement(child, { tabId: tabId || i, key: label, className }) :
              null; // No content for custom elements
          })}
        </TabContent>
      </Fragment>
    );
  }
}

export const Tab = ({ children, tabId, className }) => (
  <TabPane tabId={tabId} className={className}>
    {children}
  </TabPane>
);

Tab.propTypes = {
  children: PropTypes.node.isRequired,
  tabId: PropTypes.oneOfType([
    PropTypes.string,
    PropTypes.number,
  ]),
  className: PropTypes.string,
};

Tab.defaultProps = {
  tabId: null,
  className: null,
};
