import React from 'react';
import { Tabs, Tab } from './tabs';
import { isMobile } from '../../helpers/ua';

jest.useFakeTimers();
jest.mock('../../helpers/ua');

global.requestAnimationFrame = jest.fn();

describe('<Tabs />', () => {
  describe('with uppercase labels', () => {
    test('should match snapshot', () => {
      const wrapper = shallow((
        <Tabs upper>
          <Tab label="test">tab content 1</Tab>
          <Tab label="test">tab content 2</Tab>
        </Tabs>
      ));

      expect(wrapper).toMatchSnapshot();
    });
  });

  describe('with border', () => {
    let wrapper;
    let instance;

    beforeEach(() => {
      wrapper = shallow((
        <Tabs border>
          <Tab label="test">tab content 1</Tab>
          <Tab label="test">tab content 2</Tab>
        </Tabs>
      ));

      instance = wrapper.instance();
      instance.containerRef = { offsetWidth: 400 };
      instance.innerRef = { scrollWidth: 600 };
      instance.isSlideAvailable$ = {
        subscribe: jest.fn(),
        unsubscribe: jest.fn(),
      };
      isMobile.mockReturnValue(false);
    });

    test('should match snapshot', () => {
      expect(wrapper).toMatchSnapshot();
    });

    test('when is mobile should match snapshot', () => {
      isMobile.mockReturnValue(true);
      wrapper = shallow((
        <Tabs border>
          <Tab label="test">tab content 1</Tab>
          <Tab label="test">tab content 2</Tab>
        </Tabs>
      ));

      expect(wrapper).toMatchSnapshot();
    });

    test('should unsubscribe of isSlideAvailable on componentWillUnmount', () => {
      instance.componentWillUnmount();
      expect(instance.isSlideAvailable$.unsubscribe).toHaveBeenCalled();
    });

    describe('onResize', () => {
      beforeEach(() => {
        jest.spyOn(instance, 'isSlide');
        jest.spyOn(instance, 'slideRight');
      });

      afterEach(() => {
        instance.isSlide.mockRestore();
        instance.slideRight.mockRestore();
      });

      test('should call isSlide', () => {
        instance.onResize();
        expect(instance.isSlide).toHaveBeenCalled();
      });

      test('should not call slideRight when isMaxOffset=false', () => {
        instance.onResize();
        expect(instance.slideRight).not.toHaveBeenCalled();
      });

      test('should call slideRight when isMaxOffset=true', () => {
        instance.isMaxOffset = true;
        instance.onResize();
        expect(instance.slideRight).toHaveBeenCalled();
      });
    });

    describe('isSlide', () => {
      beforeEach(() => {
        jest.spyOn(instance, 'isSlideLeft').mockReturnValue(true);
        jest.spyOn(instance, 'isSlideRight').mockReturnValue(false);
        wrapper.setState({ offset: -100 });
      });

      afterEach(() => {
        instance.isSlideLeft.mockRestore();
        instance.isSlideRight.mockRestore();
      });

      test('should call isSlideLeft', () => {
        instance.isSlide();
        expect(instance.isSlideLeft).toHaveBeenCalledWith(-100);
        expect(wrapper.state().isSlideLeft).toBe(true);
      });

      test('should call isSlideRight', () => {
        instance.isSlide();
        expect(instance.isSlideRight).toHaveBeenCalledWith(-100);
        expect(wrapper.state().isSlideRight).toBe(false);
      });
    });

    describe('getContainerRef', () => {
      test('should set containerRef', () => {
        instance.getContainerRef({ offsetWidth: 200 });
        expect(instance.containerRef).toEqual({ offsetWidth: 200 });
      });
    });

    describe('getInnerRef', () => {
      test('should set innerRef', () => {
        instance.getInnerRef({ scrollWidth: 200 });
        expect(instance.innerRef).toEqual({ scrollWidth: 200 });
      });
    });

    describe('getMaxScrollValue', () => {
      test('should return correct value', () => {
        expect(instance.getMaxScrollValue()).toBe(-200);
      });
    });

    describe('isSlideLeft', () => {
      test('should return true', () => {
        expect(instance.isSlideLeft(-10)).toBe(true);
      });

      test('should return false', () => {
        expect(instance.isSlideLeft(0)).toBe(false);
      });
    });

    describe('isSlideRight', () => {
      test('should return true', () => {
        expect(instance.isSlideRight(-50)).toBe(true);
      });

      test('should return false', () => {
        expect(instance.isSlideRight(-650)).toBe(false);
      });

      test('should return false', () => {
        instance.containerRef = { offsetWidth: 600 };
        instance.innerRef = { scrollWidth: 400 };
        expect(instance.isSlideRight(-10)).toBe(false);
      });

      test('should return false when no containerWidth or no scrollWidth', () => {
        instance.containerRef = null;
        instance.innerRef = null;
        expect(instance.isSlideRight(0)).toBe(false);
      });
    });

    describe('getNextOffset', () => {
      test('should return correct offset when dir is 1', () => {
        expect(instance.getNextOffset({ offset: -200 }, 1)).toEqual(-40);
      });

      test('should return correct offset when dir is -1', () => {
        expect(instance.getNextOffset({ offset: -10 }, -1)).toEqual(-170);
      });
    });

    describe('prepareNextOffset', () => {
      test('should return current offset', () => {
        expect(instance.prepareNextOffset(-10)).toEqual(-10);
        expect(instance.isMaxOffset).toBe(false);
      });

      test('should return maxScrollValue when offset <= maxScrollValue', () => {
        expect(instance.prepareNextOffset(-600)).toEqual(-200);
        expect(instance.isMaxOffset).toBe(true);
      });

      test('should return 0 when offset > 0', () => {
        expect(instance.prepareNextOffset(10)).toEqual(0);
        expect(instance.isMaxOffset).toBe(false);
      });
    });

    describe('slide', () => {
      test('should return correct offset when dir is 1', () => {
        wrapper.setState({ offset: -180 });
        instance.slide(1);
        expect(wrapper.state().offset).toEqual(-20);
      });

      test('should return correct offset when dir is -1', () => {
        wrapper.setState({ offset: -10 });
        instance.slide(-1);
        expect(wrapper.state().offset).toEqual(-170);
      });
    });

    describe('slideRight', () => {
      test('should call slide with -1', () => {
        jest.spyOn(instance, 'slide');
        instance.slideRight();
        expect(instance.slide).toHaveBeenCalledWith(-1);
        instance.slide.mockRestore();
      });
    });

    describe('slideLeft', () => {
      test('should call slide with -1', () => {
        jest.spyOn(instance, 'slide');
        instance.slideLeft();
        expect(instance.slide).toHaveBeenCalledWith(1);
        instance.slide.mockRestore();
      });
    });

    describe('onWheel', () => {
      const preventDefault = jest.fn();

      beforeEach(() => {
        jest.spyOn(instance, 'prepareNextOffset');
        isMobile.mockReturnValue(true);
      });

      afterEach(() => {
        instance.prepareNextOffset.mockRestore();
        isMobile.mockReturnValue(false);
      });

      test('should not call preventDefault on desktop', () => {
        isMobile.mockReturnValue(false);
        instance.onWheel({ preventDefault, deltaX: -10 });
        expect(preventDefault).not.toHaveBeenCalled();
      });

      test('should not call requestAnimationFrame when deltaX < 0 && no isSlideLeft', () => {
        jest.spyOn(instance, 'isSlideLeft').mockReturnValue(false);
        instance.onWheel({ preventDefault: jest.fn(), deltaX: -10 });
        expect(global.requestAnimationFrame).not.toHaveBeenCalled();
        instance.isSlideLeft.mockRestore();
      });

      test('should not call requestAnimationFrame when deltaX > 0 && no isSlideRight', () => {
        jest.spyOn(instance, 'isSlideRight').mockReturnValue(false);
        instance.onWheel({ preventDefault: jest.fn(), deltaX: 10 });
        expect(global.requestAnimationFrame).not.toHaveBeenCalled();
        instance.isSlideRight.mockRestore();
      });

      test('should call requestAnimationFrame when deltaX > 0 && isSlideRight', () => {
        jest.spyOn(instance, 'isSlideRight').mockReturnValue(true);
        instance.onWheel({ preventDefault: jest.fn(), deltaX: 10 });
        expect(global.requestAnimationFrame).toHaveBeenCalled();
        instance.isSlideRight.mockRestore();
      });
    });
  });

  describe('with flex items', () => {
    test('should match snapshot', () => {
      const wrapper = shallow((
        <Tabs flexItems>
          <Tab label="test">tab content 1</Tab>
          <Tab label="test">tab content 2</Tab>
        </Tabs>
      ));

      expect(wrapper).toMatchSnapshot();
    });
  });

  describe('without tabId on Tab', () => {
    describe('without activeTab the first tab should to be active', () => {
      test('should match snapshot', () => {
        const wrapper = shallow((
          <Tabs>
            <Tab label="test">tab content 1</Tab>
            <Tab label="test">tab content 2</Tab>
          </Tabs>
        ));

        expect(wrapper).toMatchSnapshot();
      });
    });

    describe('with activeTab the right tab should to be active', () => {
      test('should match snapshot', () => {
        const wrapper = shallow((
          <Tabs activeTab={1}>
            <Tab label="test">tab content 1</Tab>
            <Tab label="test">tab content 2</Tab>
          </Tabs>
        ));

        expect(wrapper).toMatchSnapshot();
      });
    });
  });

  describe('with tabId on Tab', () => {
    describe('without activeTab the first tab should to be active', () => {
      test('should match snapshot', () => {
        const wrapper = shallow((
          <Tabs>
            <Tab label="test" tabId="first">tab content 1</Tab>
            <Tab label="test" tabId="second">tab content 2</Tab>
          </Tabs>
        ));

        expect(wrapper).toMatchSnapshot();
      });
    });

    describe('with activeTab the right tab should to be active', () => {
      test('should match snapshot', () => {
        const wrapper = shallow((
          <Tabs activeTab="second">
            <Tab label="test" tabId="first">tab content 1</Tab>
            <Tab label="test" tabId="second">tab content 2</Tab>
          </Tabs>
        ));

        expect(wrapper).toMatchSnapshot();
      });
    });
  });

  describe('toggle', () => {
    test('should select correct tab', () => {
      const wrapper = shallow((
        <Tabs>
          <Tab label="test">tab content 1</Tab>
          <Tab label="test">tab content 2</Tab>
          <span key="looool">Not a tab!</span>
        </Tabs>
      ));

      wrapper.instance().toggle(1);
      expect(wrapper.state().activeTab).toEqual(1);
    });
  });
});

describe('<Tab />', () => {
  test('should match snapshot', () => {
    const wrapper = shallow((
      <Tab label="test" tabId="first">tab content 1</Tab>
    ));

    expect(wrapper).toMatchSnapshot();
  });
});
